<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use App\Http\Controllers\ProductController;
use App\Traits\{AppConstants, MagicConstants};
use Illuminate\Support\Facades\{DB, Log, Mail, Http};
use App\Models\{ShopifyProduct, ShopifyProductSync, ShopifyProductMetafield}; 
use Exception;

class NewProductsSynCommand extends Command
{
    use MagicConstants, AppConstants;
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:new-products-syn-command';

    /**
     * The console New Product sync.
     *
     * @var string
     */
    protected $description = 'New Product sync';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        Log::info("started => New Product sync");
        $productController =  new ProductController();
        try {
            $products       = ShopifyProduct::where('is_new', 1)->where('deleted', 0)->get();
            $manual_update  = [];

            foreach ($products as $product) {
                $webhookContent = json_encode($product, JSON_HEX_APOS);
                $jsondata       = str_replace(["\n", "\r"], '', utf8_encode($webhookContent));
                $webhookChild   = utf8_encode($jsondata);

                if ($product->product_sku) {
                    $existingPart = ShopifyProduct::where('is_new', 0)
                        ->where('deleted', 0)
                        ->where('product_sku', $product->product_sku)
                        ->where('product_price', '!=', '')
                        ->where('product_price', '!=', '0.00')
                        ->first();

                    if ($existingPart) {
                        $parentJson         = $this->fetchProductJson($existingPart->product_id);
                        $parentMetas        = $this->fetchProductMetas($existingPart->product_id);
                        $webhookParentMetas = json_encode($parentMetas, JSON_HEX_APOS);
                        $jsondata1          = str_replace(["\n", "\r"], '', utf8_encode($webhookParentMetas));
                        $random_id          = rand(100, 999);

                        ShopifyProductSync::create([
                            'child_product'     => json_encode($webhookChild, JSON_UNESCAPED_SLASHES),
                            'parent_product'    => json_encode($parentJson, JSON_UNESCAPED_SLASHES),
                            'parent_metas'      => json_encode($jsondata1, JSON_UNESCAPED_SLASHES),
                            'shopifydata_id'    => $random_id,
                            'is_new'            => 1,
                        ]);

                        $product->update(['is_new' => 0]);
                    } else {
                        $manual_update[] = $product->product_title;
                        $product->update(['is_new' => 0]);
                    }
                }
            }

            $this->emailTrigger($manual_update);              
        } catch (\Exception $e) {
            $productController->errorLog($e, "Pureflow New Product sync");
            report($e);
        }
        Log::info("end => Pureflow New Product sync");
    }
    
    protected function fetchProductJson($parent_product_id)
    {
        $productController =  new ProductController();
        try {
            $url        = $productController->getStoreUrl();
            $url        = $url."/products/$parent_product_id.json"; 
            $response = Http::withBasicAuth(env('SHOPIFY_KEY_1'), env('SHOPIFY_SECREAT_1'))->get($url);

            if ($response->failed()) {
                throw new Exception($response->body());
            }

            $data_json = $response->json();
            return json_encode($data_json['product'], JSON_HEX_APOS);
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
    }

    protected function fetchProductMetas($parent_product_id)
    {
        $productController =  new ProductController();

        try {
            $url        = $productController->getStoreUrl();
            $url        = $url . "/products/$parent_product_id/metafields.json";
            $response = Http::withBasicAuth(env('SHOPIFY_KEY_1'), env('SHOPIFY_SECREAT_1'))->get($url);

            if ($response->failed()) {
                throw new Exception($response->body());
            } 
 
            $metafields = $response->json();

            $metafields = $metafields['metafields'];
            foreach ($metafields as $val) { 
                ShopifyProductMetafield::create([
                    'product_id'    => $val['owner_id'],
                    'metafield_id'  => $val['id'],
                    'namespace'     => $val['namespace'],
                    'key'           => $val['key'],
                    'value'         => $val['value'],
                    'value_type'    => $val['type'],
                ]);
            } 

            return $metafields;
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
    }

    protected function emailTrigger($emailPartNumber)
    {
        try {
            if (!empty($emailPartNumber)) {
                $partNumber = implode(',', $emailPartNumber);
                $msg = "Hi,

                New Products have been added in Pureflowair.
                Please update the price and quantity for the same.
                New products: \n $partNumber

                Thank you,
                PGI Team";

                $recipients = config("app.email_ids.new_sync_product");

                Mail::raw($msg, function ($message) use ($recipients) {
                    $message->to($recipients)
                        ->subject('Pureflow Manual Update Products List');
                }); 
            }
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
    }
}
